/**************************************************************************\
 *  aRT : API R - TerraLib                                                *
 *  Copyright (C) 2003-2005  LEG                                          *
 *                                                                        *
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 2 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU Lesser General Public      *
 *  License along with this library.                                      *
\**************************************************************************/

#include "aRTlayer.h"
#include "SEXPbasic.h"
#include "SEXPutils.h"

#include <TeProjection.h>
#include <TeImportRaster.h>
#include <TeDecoderMemoryMap.h>
#include <TeInitRasterDecoders.h>
#include <TeRaster.h>
#include <TeDatabaseUtils.h>

#include <stdio.h>
#include <iostream>

using namespace std;

SEXP aRTlayer::GetBox()
{
	TeRepresentation* rep = Layer()->getRepresentation(TePOLYGONS);
	if(rep)
	{
	    return SEXPutils::GetBoxTeBox(rep->box_);
	}
	return R_NilValue;
}

TeGeomRep aRTlayer::GeomRep()
{
	if( Layer()->hasGeometry(TePOLYGONS) ) return TePOLYGONS;
	if( Layer()->hasGeometry(TeLINES) )    return TeLINES;
	if( Layer()->hasGeometry(TePOINTS) )   return TePOINTS;
	
	error("Layer does not have a valid geometry\n");
	return TeGEOMETRYNONE;
}

SEXP aRTlayer::GetNearestNeighbors(SEXP options, aRTlayer* alayer)
{
	string objId = GET_STRING_ELEMENT(options, "id");
	int numRes   = GET_INTEGER_ELEMENT(options, "quantity");

	SEXP result;	
	TeGeomRep visRep = GeomRep();
	TeGeomRep actRep = alayer->GeomRep();
	string actGeomTable;
	string visGeomTable;
	visGeomTable = Layer()->tableName(visRep);
	actGeomTable = alayer->Layer()->tableName(actRep);

	string visCollTable;
	Keys visIdsOut;

	Database->nearestNeighbors(actGeomTable,
							   actRep,
							   objId,
							   visGeomTable,
							   visCollTable,
							   visRep,
							   visIdsOut,
							   numRes);
	
    PROTECT(result = allocVector(STRSXP, visIdsOut.size()));

    for(unsigned i = 0; i < visIdsOut.size(); i++)
        SET_STRING_ELT(result, i, mkChar(visIdsOut[i].c_str()));

    UNPROTECT(1);
    return result;
}

SEXP aRTlayer::GetDistance(SEXP ids, aRTlayer* alayer)
{
	SEXP result;

	double distance;
	
	TeGeomRep visRep = GeomRep();
	TeGeomRep actRep = alayer->GeomRep();
	string actGeomTable;
	string visGeomTable;
	visGeomTable = Layer()->tableName(visRep);
	actGeomTable = alayer->Layer()->tableName(actRep);

	string objId1 = CHAR(STRING_ELT(ids,0));
	string objId2 = CHAR(STRING_ELT(ids,1));

	bool right;
	right = Database->calculateDistance(actGeomTable,
					                    actRep,
										objId1,
										visGeomTable,
										visRep,
										objId2,
										distance);
   
	if(!right) error("Could not calculate the distance\n");
	PROTECT(result = allocVector(REALSXP, 1));
    REAL(result)[0] = distance;     	
	UNPROTECT(1);
	return result;
}

SEXP aRTlayer::GetRelation(SEXP options, aRTlayer* alayer)
{
	string relation = GET_STRING_ELEMENT(options, "relation");
	SEXP ids        = GetListElement(options, aRTstrId);

	if(Database != alayer->Database)
		error("The layers must be in the same database\n");

	SEXP result;
	TeGeomRep visRep = GeomRep();
	TeGeomRep actRep = alayer->GeomRep();
	string actGeomTable;
	string visGeomTable;
	visGeomTable = Layer()->tableName(visRep);
	actGeomTable = alayer->Layer()->tableName(actRep);
	int relate = TeUNDEFINEDREL;
	Keys actIdsIn, visIdsOut;

	switch(relation[0])
	{
		case 'c': relate = TeCONTAINS;   break;
		case 'o': relate = TeOVERLAPS;   break;
		case 'x': relate = TeCROSSES;    break;
		case 'd': relate = TeDISJOINT;   break;
		case 't': relate = TeTOUCHES;    break;
		case 'w': relate = TeWITHIN;     break;
		case 'i': relate = TeINTERSECTS; break;
		case 'e': relate = TeEQUALS;     break;
		case 'v': relate = TeCOVERS;     break;
		case 'y': relate = TeCOVEREDBY;  break;
		default:
			error("Relation %s inexistent\n", relation.c_str());
	}

	if(ids == R_NilValue)
	{
    	string sqlstr = "select object_id from " + actGeomTable;
	    TeDatabasePortal* portal = alayer->Database->getPortal();
		portal->query(sqlstr);

	    while( portal->fetchRow() )                              
	    {                                                        
	        string id = portal->getData(0);                             
	        actIdsIn.push_back(id);
		}
	}	
	else
	{
		for(int i = 0; i < LENGTH(ids); i++)
			actIdsIn.push_back( CHAR(STRING_ELT(ids,i)) );
	}
	
	bool right;

	right = Database->spatialRelation (actGeomTable,
			        		           actRep,
					        		   actIdsIn,
							           visGeomTable,
							           visRep,
							           visIdsOut,
							           relate);

	if(!right) error("Could not execute the spatial relation\n");
	
    PROTECT(result = allocVector(STRSXP, visIdsOut.size()));
	
	for(unsigned i = 0; i < visIdsOut.size(); i++)
    	SET_STRING_ELT(result, i, mkChar(visIdsOut[i].c_str()));     	

	UNPROTECT(1);
	return result;
}

SEXP aRTlayer::GetSetMetric(SEXP options)
{
	string metric = GET_STRING_ELEMENT(options, "metric");
	SEXP ids      = GetListElement(options, aRTstrId);
	string id1, id2;
	SEXP result, colnames;
	TeGeomRep actRep = GeomRep();
	string actGeomTable;
	TeGeometryVect geomvec;
	Keys keys;
	TePolygonSet ps;
	int i = 0;
	
	actGeomTable = Layer()->tableName(actRep);

	PROTECT(result   = allocVector(VECSXP, 1));
	PROTECT(colnames = allocVector(STRSXP, 1));

	bool good = false;
	switch(metric[0])
	{
		case 'u':
			for(i = 0; i < LENGTH(ids); i++)
				keys.push_back( CHAR(STRING_ELT(ids,i)) );
			
			good = Database->geomUnion(actGeomTable, actRep, keys, geomvec);
			break;
		case 'i':
			for(i = 0; i < LENGTH(ids); i++)
				keys.push_back( CHAR(STRING_ELT(ids,i)) );
			
			good = Database->geomIntersection(actGeomTable, actRep, keys, geomvec);
			break;
		case 'd':
			id1 = CHAR(STRING_ELT(ids,0));
			id2 = CHAR(STRING_ELT(ids,1));
			cout << id1.c_str() << "  " << id2.c_str() << endl;
			good = Database->geomDifference(actGeomTable, actRep, id1, id2, geomvec);
			break;
		case 'x':
			id1 = CHAR(STRING_ELT(ids,0));
			id2 = CHAR(STRING_ELT(ids,1));
			cout << id1.c_str() << "  " << id2.c_str() << endl;

			good = Database->geomXOr(actGeomTable, actRep, id1, id2, geomvec);
			break;
		default:
			error("Invalid set metric: %s\n", metric.c_str());
	}

	if(!good) error("Operation could not be done\n");

    TeGeometryVect::iterator it = geomvec.begin();
    while (it!= geomvec.end())
    {
        TeGeometry* geom = (*it);

		// "spatialQuery.ui.h" line 1550
		if (dynamic_cast<TePolygonSet*> (geom))
        {
			TePolygonSet polySet, *polSet;
            polSet = new TePolygonSet();
            polSet = (TePolygonSet*) geom;
            polySet = *polSet;

			SEXP other;
			//SET_STRING_ELT(colnames, 0, mkChar(aRTstrPolygon));
			
			PROTECT(other = allocVector(VECSXP,1 ));

			SET_VECTOR_ELT(other, 0, GetPolygonsTePolygonSet(polySet));
		    SET_STRING_ELT(colnames, 0, mkChar(aRTstrPolygon));       
                                                             
		    setAttrib(result, R_NamesSymbol, colnames);              
                                                             
		    SET_VECTOR_ELT(result, 0, other);
			UNPROTECT(3);
			return result;
		}
		else
		{
			error("Type returned not implemented\n");
		}

		it++;
	}

	return R_NilValue;	
}

SEXP aRTlayer::GetMetric(SEXP options)
{
	string metric = GET_STRING_ELEMENT(options, "metric");

	SEXP result, colnames, ids, other;
	TeGeomRep actRep = GeomRep();
	string actGeomTable;
	Keys keys;
	TeRepresPointerVector representations;
	TePolygonSet ps;
	TePointSet cs;
	TeBox box;
	double value;
	string id;
	int i = 0;
	
	actGeomTable = Layer()->tableName(actRep);

	string sqlstr = "select object_id from " + actGeomTable;
	TeDatabasePortal* portal = Database->getPortal();
	portal->query(sqlstr);

	// para que serve o getRepresentation??
//	Layer()->getRepresentation (actRep, representations);
//	cout << "size: " << representations.size() << endl;
	// this is not working...	
//	cout << "id: " << representations[0]->id_ << endl;
//	stringstream str;
//	str << representations[0]->id_;

	PROTECT(result   = allocVector(VECSXP, 2));
	PROTECT(colnames = allocVector(STRSXP, 2));
	PROTECT(ids = allocVector(STRSXP, portal->numRows()));

#define METRIC_FUNCTION(type_, metric_call_, set_call_, attrib_name_ , clear_call_)\
    PROTECT(other = allocVector(type_, portal->numRows()));  \
    while( portal->fetchRow() )                              \
    {                                                        \
        id = portal->getData(0);                             \
        keys.clear();                                        \
        keys.push_back(id);                                  \
        clear_call_;                                         \
        Database->metric_call_;                              \
        set_call_;                                           \
        SET_STRING_ELT(ids, i, mkChar(id.c_str()));          \
        i++;                                                 \
    }                                                        \
    SET_STRING_ELT(colnames, 0, mkChar(aRTstrId));           \
    SET_STRING_ELT(colnames, 1, mkChar(attrib_name_));       \
                                                             \
    setAttrib(result, R_NamesSymbol, colnames);              \
                                                             \
	SET_VECTOR_ELT(result, 0, ids);                          \
    SET_VECTOR_ELT(result, 1, other);                        \
    UNPROTECT(4);                                            \
    return result;

	switch(metric[0])
	{
		case 'a':
			METRIC_FUNCTION(REALSXP,
							calculateArea(actGeomTable, actRep, keys, value),
							REAL(other)[i] = value,
							"area",)
		case 'l':
			METRIC_FUNCTION(REALSXP,
							calculateLength(actGeomTable, actRep, keys, value),
							REAL(other)[i] = value,
							"length",)
		case 'c':
			METRIC_FUNCTION(VECSXP,
							Centroid(actGeomTable, actRep, cs, keys),
							SET_VECTOR_ELT(other, i, GetPointsTePointSet(cs)),
							aRTstrPoint,
							cs.clear())
			break;
		case 'b':
			// pega o buffer interno e externo. Lubia: apenas um dos dois?
			PrintSilent("This function can be slow\n");
			value = GET_REAL_ELEMENT(options, "dist");
			METRIC_FUNCTION(VECSXP,
							Buffer(actGeomTable, actRep, keys, ps, value),
							SET_VECTOR_ELT(other, i, GetPolygonsTePolygonSet(ps)),
							aRTstrPolygon,
							ps.clear())
		case 'h':
			METRIC_FUNCTION(VECSXP,
							ConvexHull(actGeomTable, actRep, keys, ps),
							SET_VECTOR_ELT(other, i, GetPolygonsTePolygonSet(ps)),
							aRTstrPolygon,
							ps.clear())
			break;
		case 'r':
			// pegar a coluna da tabela onde tem geometria. Lubia?
			//Database->getMBRGeom (actGeomTable, "1", box);//, string colGeom)
			return GetBoxTeBox(box);
		default:
			error("Invalid metric: %s\n", metric.c_str());
	}
	return R_NilValue;
}

void aRTlayer::CreateEmptyTable()
// creates an empty table and add rows with the object's ids
{
    TeAttributeList attList;
    TeAttribute at;

    at.rep_.type_ = TeSTRING;
    at.rep_.numChar_ = 16;
    at.rep_.name_ = "object_id";
    at.rep_.isPrimaryKey_ = true;

    attList.push_back(at);

    TeTable attTable( LayerName,  attList, "object_id", "object_id", TeAttrStatic);
    if ( !Layer()->createAttributeTable(attTable) )
    {
        stringstream s;
        s << "Error: " << Layer()->database()->errorMessage() << endl;
        error(StreamToChar(s));
    }

	Layer()->addAttributeTable(attTable); //<== TODO:precisa??? esta funcionando sem isso
    attList.clear();
    at.rep_.name_ = "tempCol";
    at.rep_.type_ = TeSTRING;
    at.rep_.numChar_ = 100;
    at.rep_.isAutoNumber_ = false;
    at.rep_.isPrimaryKey_ = false;
    attList.push_back(at);

    if(Database->tableExist("tempTable"))
        if(Database->execute("DROP TABLE tempTable") == false)
			error("Fail to drop the temporary table!");

    if(Database->createTable("tempTable", attList) == false)
		error("Fail to create the temporary table!");

    string popule, geo;
    if(Layer()->hasGeometry(TePOLYGONS))
    {
        geo = Layer()->tableName(TePOLYGONS);
        popule = "INSERT INTO tempTable (tempCol) SELECT object_id FROM " + geo;
        if(Database->execute(popule) == false)
			error("Fail to drop the temporary table!");
    }
    if(Layer()->hasGeometry(TePOINTS))
    {
        geo = Layer()->tableName(TePOINTS);
        popule = "INSERT INTO tempTable (tempCol) SELECT object_id FROM " + geo;
        if(Layer()->database()->execute(popule) == false)
			error("Fail to drop the temporary table!");
    }
    popule = "INSERT INTO " + LayerName + " (object_id_) SELECT DISTINCT tempCol FROM tempTable";
    if(Database->execute(popule) == false)
    { // this two errors cant exist, only the first will be called
		error("Fail to insert the objects in the table!");
        if(Database->execute("DROP TABLE tempTable") == false)
			error("Fail to drop the temporary table!");
    }

    if(Database->execute("DROP TABLE tempTable") == false)
		error("Fail to drop the temporary table!");
	
    PrintSilent("Table created successfully!\n");
}

SEXP aRTlayer::GetRaster()
{
    if( !(Layer()->geomRep() & TeRASTER) ) error("Theme does not have raster");
      TeRaster* raster = Layer()->raster();

    TeDecoderSEXP* decoder = new TeDecoderSEXP( raster->decoder() );
    return decoder->data();
}

SEXP aRTlayer::GetGeometry()   { return SEXPutils::GetLayerGeometry  ( Layer() ); }
SEXP aRTlayer::GetData()       { return SEXPutils::GetLayerData      ( Layer() ); }
SEXP aRTlayer::GetAttributes() { return SEXPutils::GetLayerAttributes( Layer() ); }

vector<TeColor> generateColors(int numColors, bool color)
{
    int i;
    vector<TeColor> colorVec;

    if(color)
    {
        TeColor cor(200, 255, 255); // cyan light
        for(i=0; i<50; i++) // cyan light to cyan
        {
            colorVec.push_back(cor);
            if((cor.red_ -= 4) < 0)
                cor.red_ = 0;
        }
        for(i=0; i<128; i++)    // cyan to green
        {
            colorVec.push_back(cor);
            cor.blue_--;
            cor.blue_--;
            if(cor.blue_ < 0)
                cor.blue_ = 0;
        }
        for(i=0; i<128; i++) // green to yellow
        {
            colorVec.push_back(cor);
            cor.red_++;
            cor.red_++;
            if(cor.red_ > 255)
                cor.red_ = 255;
        }
        for(i=0; i<256; i++) // yellow to orange to red
        {
            colorVec.push_back(cor);
            cor.green_--;
            if(cor.green_ < 0)
                cor.green_ = 0;
        }
    }
    else
    {
        TeColor cor(223, 223, 223); // white
        for(i=0; i<200; i++)    // white to black
        {
            colorVec.push_back(cor);
            cor.red_--;
            cor.green_--;
            cor.blue_--;
        }
    }
    double step = (double)(colorVec.size()) / (double)numColors;
    vector<TeColor> colorRet;
    for(i=0; i<numColors; i++)
    {
        int ind = (int)(step * (double)i + .5);
        if(ind > (int)(colorVec.size() - 1))
            ind = colorVec.size() - 1;
        colorRet.push_back(colorVec[ind]);
    }

    return colorRet;
}

void aRTlayer::AddColumn(SEXP data)
{
	SEXP name     = GetListElement(data, "name_");
    SEXP elements = GetListElement(data, CHAR(STRING_ELT(name,0)));
    SEXP ids      = GetListElement(data, "id");
    SEXP size     = GetListElement(data, "size_");
    SEXP type     = GetListElement(data, "type_");

	TeTable table;
    if( !Database->loadTable(LayerName, table) )
        error("Could not load the table!\n");

	TeAttributeRep atrep;
 	
	atrep.name_ =  CHAR(STRING_ELT(name,0));

    switch( CHAR(STRING_ELT(type,0))[0] )
    {
        case 'i': case 'd':
            atrep.type_ = TeINT;
            break;
        case 's': case 'c':// if there is only one string we will use $size to the sizes
            atrep.type_ = TeSTRING;
            atrep.numChar_ = (int) REAL(size)[0];
            break;
        case 'f':  case 'r':
            atrep.type_ = TeREAL;
            break;
        default:
            error("Invalid attribute type.\n");
	}

	Database->addColumn(LayerName, atrep);
	string update;
	int length = LENGTH(elements);
	
	for(int i = 0; i < length; i++)
	{
		update = "UPDATE " +LayerName+" SET "+atrep.name_+" = ";
	    switch (atrep.type_)
	    {
	        case TeSTRING:
	            update += "'";
				update += CHAR(STRING_ELT(elements, i));
				update += "'";
	            break;
			case TeINT:
		      {
				stringstream s;
				s.clear();
				s << INTEGER(elements)[i];
				update += StreamToChar(s);
			  }
				break;
	        default:
			  {
				stringstream s;
				s.clear();
				s << REAL(elements)[i];
			    update += StreamToChar(s);
			  }
	            break;
	    }

	    update += " WHERE object_id_  = '";
		update += CHAR(STRING_ELT(ids, i));
		update += "'";

    	if(!Database->execute(update))
		{
			cout << update << endl;
	        error("query not executed");
		}
	} 
	Layer()->loadLayerTables();
	PrintSilent("data inserted in the database\n");
}

void aRTlayer::AddRaster(SEXP sraster)
{
	if(!SEXPutils::AddRaster(Layer(), sraster))
		error("Could not import the raster");

	PrintSilent("Raster saved successfully\n");
	Layer()->loadLayerTables();
	//Database -> loadLayerSet();
}

aRTlayer::aRTlayer(TeDatabase* database, string layername)
{
	Database = database;
	LayerName = layername;
}

aRTlayer::~aRTlayer()
{
	PrintSilent("Deleting aRTlayer \"%s\" from memory\n", LayerName.c_str());
	Clear();
}
	
void aRTlayer::AddPoints(SEXP sps)
{
	if(!Layer()->geomRep() | TePOINTS)
		Layer()->addGeometry(TePOINTS);
	PrintSilent("converting...\n");
	
	TePointSet ps = SEXPutils::GetTePointSet(sps);
	PrintSilent("adding...\n");
	if( !Layer()->addPoints(ps) ) error("Points could not be added\n");
	stringstream s;

	s << ps.size() << " points were added to the layer\n";
	PrintSilent(StreamToChar(s));
	Layer()->loadLayerTables();
}

void aRTlayer::AddPolygons(SEXP polset)
{
   if(!Layer()->geomRep() | TePOLYGONS)
        Layer()->addGeometry(TePOLYGONS);
	cout<< "converting..." << endl;

	TePolygonSet ps = SEXPutils::GetTePolygonSet(polset);
	PrintSilent("adding...\n");
	if( !Layer()->addPolygons(ps) ) error("Polygons could not be added!\n");
	stringstream s;

	s << ps.size() << " polygons were added to the layer\n";
	PrintSilent(StreamToChar(s));
	Layer()->loadLayerTables();
}

void aRTlayer::AddLines(SEXP lineset)
{
   if(!Layer()->geomRep() | TeLINES)
        Layer()->addGeometry(TeLINES);
	cout<< "converting..." << endl;

	TeLineSet ls = SEXPutils::GetTeLineSet(lineset);
	PrintSilent("adding...\n");
	if( !Layer()->addLines(ls) ) error("Lines could not be added!\n");
	stringstream s;

	s << ls.size() << " lines were added to the layer\n";
	PrintSilent(StreamToChar(s));
	Layer()->loadLayerTables();
}

// TODO: adicionar bool append = false
void aRTlayer::AddElements(string tablename, SEXP data)
{
	const int numcols = LENGTH(data);
	TeTable table;
	int pos;
	SEXP vars, attribs;
	TeTableRow row;
	
	if(!numcols) error("Impossible add a table with no columns!");

	attribs = getAttrib(data, R_NamesSymbol);
    char *label = CHAR(STRING_ELT(attribs, 0));
    vars = GetListElement(data, label);
	const int numlines = LENGTH(vars);
	
	if( !Database->loadTable(tablename, table) )
		error("Could not load the table!\n");
	
	TeAttributeList attlist = table.attributeList();
	vector<int> references;

	// calculate the order of the data to be inserted, because it can be different from the table order
	// the order to be inserted in the TeTableRow will be referred by _references_
	for(TeAttributeList::iterator it = attlist.begin(); it != attlist.end(); it++)
	{
		pos = -1;
		for(int j = 0; j != numcols && pos == -1; j++)
		{
			cout << string(CHAR(STRING_ELT(attribs, j))) << endl;
			if(it->rep_.name_ == string(CHAR(STRING_ELT(attribs, j))).c_str())
				pos = j;
		}

		if(pos == -1)
		{
			stringstream s;
			s << "Attribute \"" << it->rep_.name_.c_str() << "\" not found!" << endl;
			error(StreamToChar(s));
		}
		references.push_back(pos);
	}
	
	// insert the data into the table
	for(int i = 0; i != numlines; i++)
	{
		for(unsigned j = 0; j < references.size(); j++)
		{
			stringstream s;
			if(isReal(VECTOR_ELT(data, references[j])))
				s << REAL(VECTOR_ELT(data, references[j]))[i];
			else if(isInteger(VECTOR_ELT(data, references[j])))
				s << INTEGER(VECTOR_ELT(data, references[j]))[i];
			else if(isString(VECTOR_ELT(data, references[j])))
				s << CHAR( STRING_ELT( VECTOR_ELT(data, references[j]), i) );
			else
				error("ERROR: type not supported!");
			
			row.push_back( s.str() );
		}
        table.add(row);
        row.clear();
	}

	if(!Layer()->saveAttributeTable(table))
		//(!Layer->database()->updateTable(table))
		error("Could not update the table\n");
	PrintSilent("Table updated successfully\n");
}

void aRTlayer::CreateTable(string Tablename, SEXP attributes, int keyposition)
{
	SEXP att_names = GetListElement(attributes, "name");
	SEXP att_types = GetListElement(attributes, "type");
	const int quant = LENGTH(att_names);
	SEXP att_sizes = R_NilValue;
	TeAttributeList attList;
    TeAttribute at;
	TeTableRow row;
	Rprintf("kp = %d\n",keyposition);
	const string object_id_name = CHAR(STRING_ELT(att_names,keyposition));
	for(int i = 0; i < quant; i++)
	{
		switch( CHAR(STRING_ELT(att_types,i))[0] )
		{
			case 'i': case 'd':
				at.rep_.type_ = TeINT;
				break;
			case 's': // if there is only one string we will use $size to the sizes
				at.rep_.type_ = TeSTRING;
				if(att_sizes == R_NilValue) att_sizes = GetListElement(attributes, "size");
				at.rep_.numChar_ = (int) REAL(att_sizes)[i];
				break;
			case 'f':  case 'r':
				at.rep_.type_ = TeREAL;
				break;
			default:
				error("Invalid attribute type.\n");
		}
		at.rep_.name_ = CHAR(STRING_ELT(att_names,i));
		cout << "name = " << at.rep_.name_ << endl;
	    at.rep_.isPrimaryKey_ = (keyposition == i);
		if(keyposition == i) cout << "kp == " << i << endl;
		attList.push_back(at);
	}
	
	TeTable attTable(Tablename, attList, object_id_name, object_id_name);
    if (!Layer()->createAttributeTable(attTable))
	{
		stringstream s;
		s << "Error: " << Database->errorMessage() << endl;
		error(StreamToChar(s));
	}
	
	PrintSilent("Table created successfully!\n");
}

// 0 <= keyposition < LENGTH(table)
void aRTlayer::AddTableData(string strTablename, SEXP table, int keyposition)
// if key data on the table is not string, it will be converted to string
{
	TeAttributeList attList;
    TeAttribute at;
	int i;
	//int numberofattr[LENGTH(table)];
	//int sizeofattr[LENGTH(table)];
	TeTableRow row;
	//int typesofattr[LENGTH(table)]; // TeINT, TeSTRING, etc.
	
	for(int i = 0; i < 10; i++);
	
	for(i = 0; i < 10; i++)
	{
		at.rep_.type_ = TeSTRING;
		at.rep_.numChar_ = 16;
		at.rep_.name_ = "object_id";
		at.rep_.isPrimaryKey_ = (keyposition == i);
		attList.push_back(at);
	}
	
	TeTable attTable(strTablename, attList, "object_id", "object_id"); // creates a table in memory

	for(int i = 0; i < 10; i++)
	{
        row.push_back("reta");
        row.push_back("An straight line");
        row.push_back("11.1");
        row.push_back("11");
        attTable.add(row);
        row.clear();
	}

	if (!Layer()->createAttributeTable(attTable))
        cout << "Error: "  << endl;//<< db_->errorMessage() << endl;
	if (!Layer()->saveAttributeTable( attTable ))	error("Error trying to save the attribute table");
}

// this is the order to appear inside R. S is from Summary
enum aRTlayerSenum
{
	aRTlayerSClass = 0,
	aRTlayerSLayer,
	aRTlayerSPolygons,
	aRTlayerSLines,
	aRTlayerSPoints,
	aRTlayerSRaster,
	aRTlayerSProjection,
	aRTlayerSDatum,
	aRTlayerSLongitude,
	aRTlayerSLatitude,
	aRTlayerSSize
};
					   
SEXP aRTlayer::Summary()
{
        SEXP info, names;

        info    = allocVector(VECSXP, aRTlayerSSize);  // List containing the connection attributes
        names   = allocVector(STRSXP, aRTlayerSSize);  // Names Vector

		string raster = "no";
	   if( Layer()->geomRep() & TeRASTER) raster = "yes";
		
	   	STRING_LIST_ITEM(info, names, aRTlayerSRaster,     "raster",     raster);
    	STRING_LIST_ITEM(info, names, aRTlayerSClass,      "class",      Class());
	    STRING_LIST_ITEM(info, names, aRTlayerSProjection, "projection", Layer()->projection()->name());
		STRING_LIST_ITEM(info, names, aRTlayerSLayer,      "layer",      LayerName);
		STRING_LIST_ITEM(info, names, aRTlayerSDatum,      "datum",      Layer()->projection()->datum().name());
		
        DOUBLE_LIST_ITEM(info, names, aRTlayerSLongitude,  "longitude",  Layer()->projection()->lon0());
        DOUBLE_LIST_ITEM(info, names, aRTlayerSLatitude,   "latitude",   Layer()->projection()->lat0());
        DOUBLE_LIST_ITEM(info, names, aRTlayerSPolygons,   "polygons",   Layer()->nGeometries(TePOLYGONS));
        DOUBLE_LIST_ITEM(info, names, aRTlayerSPoints,     "points",     Layer()->nGeometries(TePOINTS));
        DOUBLE_LIST_ITEM(info, names, aRTlayerSLines,      "lines",      Layer()->nGeometries(TeLINES));

		setAttrib(info, R_NamesSymbol, names); // Set the names of the list

        return info;
}

const string aRTlayer::Print()
{
	stringstream s;

	if (!IsValid()) error("Invalid object!");
		 
	 s << artOBJECT("aRTlayer")     << "\n\n"
	   << "Layer = \""              << LayerName                             << "\"" << endl
       << "Number of polygons = "   << Layer()->nGeometries(TePOLYGONS)      << endl
       << "Number of lines = "      << Layer()->nGeometries(TeLINES)         << endl
       << "Number of points = "     << Layer()->nGeometries(TePOINTS)        << endl;

	 if(Layer()->hasGeometry(TeRASTER))
		s << "Layer has raster data" << endl;
	 else
		s << "Layer does not have raster data" << endl;
	 
	 if (Layer() -> projection() == NULL)
     {
          s << "Layer has no projection!" << endl;
		  return s.str();
	 }

	 s << "Projection Name = \""   << Layer()->projection()->name()         << "\""<< endl
	   << "Projection Datum = \""  << Layer()->projection()->datum().name() << "\""<< endl
	   << "Projection Longitude = "<< Layer()->projection()->lon0()                << endl
	   << "Projection Latitude = " << Layer()->projection()->lat0()                << endl;

	return s.str();
}

SEXP aRTlayer::ShowTables()
{
	SEXP vectorTables;
	TeAttrTableVector atts;
	TeAttributeList attrList;
	
	if (!Layer() -> getAttrTables(atts)) error("Error loading tables from layer!");

	vectorTables = allocVector(STRSXP, atts.size());

	for (unsigned i = 0; i < atts.size(); i++)
		SET_STRING_ELT(vectorTables, i, mkChar(atts[i].name().c_str()));

	return vectorTables;
}

aRTcomponent* aRTlayer::Open(SEXP options)
{
	string theme_name = GET_STRING_ELEMENT(options, "themename");
	bool create       = GET_BOOL_ELEMENT  (options, "create");
	string viewname   = GET_STRING_ELEMENT(options, "viewname");
	SEXP tableNames   = GetListElement    (options, "tables");
	SEXP colors       = GetListElement    (options, "colors");

	if( TeUtils::GetThemeFromDatabase(Database, theme_name) )
	{
		if(create) warning("Trying to create an existent theme\n");
		PrintSilent("Opening an existent theme\n");
		TeUtils::GetThemeFromDatabase(Database, theme_name)->loadThemeTables();
		aRTtheme* child = new aRTtheme(Database, theme_name);
		Parent->AddChild(child);
		return child;
	}
	
	if(!create)
	    error("The theme does not exist! If you want to create set create=TRUE\n");
		
	PrintSilent("Creating a new theme\n");
    TeTheme* te_theme = new TeTheme( theme_name, Layer() );

    // try to create a view. no problem if it already exists.
	((aRTdb*)Parent)->CreateView(viewname);

	// selecting the tables
	TeLayer* layer = Layer();
	vector<string> attrTableNames;
	TeTable aux;
	TeAttrTableVector atts;
	tableNames = AS_CHARACTER(tableNames);
	
	for (int i = 0; i < length(tableNames); i++)
	{
	    string tableName = CHAR(STRING_ELT(tableNames, i));
	    if (!layer -> getAttrTablesByName(tableName, aux))
	    {
	        error("Table \"%s\" inexist", tableName.c_str());
	    }
	    else
	    {
	        TeTable table(tableName);
	        Database -> loadTableInfo(table);
	        atts.push_back(table);
	    }
	}
	if( !te_theme -> setAttTables (atts) )
		error("Could not set the attribute tables");
		
	TeProjection* proj = layer -> projection();
	const string user  = layer -> database() -> user();
	TeView* view = TeUtils::GetViewFromDatabase(Database, viewname);
	if (!view)
	    error("There is no view named \'%s\' in the database!"
	          "Use \'createView\' first.\n", viewname.c_str());

	// TODO: set the projection only if it is the first theme of the view
	view -> projection(proj);
	view -> add(te_theme);
	te_theme -> visibility(true);
	int allrep = layer -> geomRep();
	te_theme -> visibleRep(allrep);

	if( Layer()->hasGeometry(TeRASTER) )
	{
		te_theme->visibleRep(TeRASTER | 0x40000000);
			
		if( !te_theme->save() )
		{
			stringstream s;
	        s << "Fail to save theme: " << Database->errorMessage() << endl;
	        error( StreamToChar(s) );
		}
		else PrintSilent("Theme saved successfully\n");

		vector<TeColor> te_colors = SEXPutils::SEXPToColors(colors);

	    TeGrouping* groupKernel = new TeGrouping();
	    groupKernel->groupMode_ = TeRasterSlicing;
	    groupKernel->groupNumSlices_ = te_colors.size();
	    groupKernel->groupPrecision_ = 8;

	    if (!te_theme->buildGrouping(groupKernel))
	    {
			stringstream s;
	        s << "Fail to generate theme: " << Database->errorMessage() << endl;
	        error( StreamToChar(s) );
	    }

	    for (int ng = 0; ng < te_theme->grouping()->groupNumSlices_; ng++)
	    {
	        TeVisual visual(TePOLYGONS);
	        visual.color(te_colors[ng]);
	        visual.transparency(te_theme->defaultLegend().visual(TePOLYGONS).transparency());
	        visual.contourStyle(te_theme->defaultLegend().visual(TePOLYGONS).contourStyle());
	        visual.contourWidth(te_theme->defaultLegend().visual(TePOLYGONS).contourWidth());
	        visual.contourColor(te_theme->defaultLegend().visual(TePOLYGONS).contourColor());
	        te_theme->setGroupingVisual(ng+1, visual, TePOLYGONS);
	    }

	    if (!te_theme->saveGrouping())
	    {
			stringstream s;
	        s << "Fail to generate theme: " << Database->errorMessage() << endl;
	        error( StreamToChar(s) );
	    }
	}
	else // not a raster theme
	{
    	PrintSilent("Saving...\n");
    	if ( !te_theme -> save() )
    	{
    	    stringstream s;
    	    s << "Error saving theme: " << Database -> errorMessage() << endl;
    	    error( StreamToChar(s) );
    	}

    	PrintSilent("Building collection...\n");
    	if ( !te_theme -> buildCollection() )
    	{
    	    stringstream s;
    	    s << "Error building theme: " << Database -> errorMessage() << endl;
    	    error( StreamToChar(s) );
    	}
	}

	PrintSilent("Saved successfully\n");

	aRTtheme* child = new aRTtheme(Database, theme_name);
	Parent->AddChild(child);
	te_theme->loadThemeTables();
	return child;
}

void aRTlayer::Populate(SEXP data)
{
  	SEXP coordX, coordY, vars, names;//, dim;
  	int numrows, numcols;

  	//First step: insert coordinates in a layer
  	TePointSet pointSet;
  	TePoint point;
  	coordX = GetListElement(data, "x");
  	coordY = GetListElement(data, "y");

  	if (isReal(coordX)) cout << "Coord X eh real" << endl;
  	if (isInteger(coordX)) cout << "Coord X eh int" << endl;
  	if (isReal(coordY)) cout << "Coord Y eh real" << endl;
  	if (isInteger(coordY)) cout << "Coord Y eh int" << endl;

  	names = getAttrib(data, R_RowNamesSymbol); // attribute row.names from data.frame
  	numrows = length(names); 
  	numcols = length(data); // macro ncols doesn't work here

  	for (int i = 0; i < numrows; i++)
	{
    	double x = REAL(coordX)[i];
    	double y = REAL(coordY)[i];
    	TeCoord2D pt(x, y);
    	char *id;
    	asprintf(&id, "%d", i);
    	point.add(pt);
    	point.objectId(id);
    	pointSet.add(point);
  	}
  	Layer() -> addPoints(pointSet);
  	pointSet.clear();

  	// Creates an attribute list
  	TeAttributeList attList;
  	TeAttribute at;

  	// create a attribute column named object_id
  	at.rep_.type_ = TeSTRING;
  	at.rep_.numChar_ = 16;
  	at.rep_.name_ = "object_id";
  	at.rep_.isPrimaryKey_ = true;
  	attList.push_back(at);

  	// return a vector with the attribute names
  	names = getAttrib(data, R_NamesSymbol);

  	// creates the remaining attribute columns
  	for (int i = 2; i < length(data); i++)
	{
    	char* label = CHAR(STRING_ELT(names, i));
    	vars = GetListElement(data, label);

    	if (isReal(vars))
		{
    		at.rep_.type_ = TeREAL;
    		cout << " real: " << endl;
    	}

    	if (isInteger(vars))
		{
    		at.rep_.type_ = TeINT;
    		cout << " inteiro: " << endl;
    	}

    	if (isString(vars))
		{
    		at.rep_.type_ = TeSTRING;
    		cout << " string: " << endl;
    	}

    	at.rep_.name_ = label;
    	at.rep_.isPrimaryKey_ = false;
    	attList.push_back(at);  
  	}

  	// stores the attribute columns in a layer
  	TeTable attTable(LayerName, attList, "object_id", "object_id");
  	if (!Layer() -> createAttributeTable(attTable))
	{
    	cout << "Error: " << Database -> errorMessage() << endl;
    	exit(0);
  	}

  	// creates a table to store the attribute data
  	TeTableRow row;
  	SEXP vars2;
  	for (int i = 0; i < 2533; i++)
  	{
    	char *id;
    	asprintf(&id, "%d", i);
    	row.push_back(id);
    	for (int j = 2; j < length(data); j++)
		{
      		char *label = CHAR(STRING_ELT(names, j));
      		vars2 = GetListElement(data, label);
      		double inc = REAL(vars2)[i];
      		//Rprintf("%f valor:   \n", inc);
      		char *value;
      		asprintf(&value, "%f", inc);
      		row.push_back(value);
    	}
    	attTable.add(row);
    	row.clear();
  	}
  
  	// Saves the table in the database
  	if (!Layer() -> saveAttributeTable( attTable )) {
    	cout << "Error: " << Database -> errorMessage() << endl;
	
  		exit(0);
  	}
  
  	//UNPROTECT(1);
  	return;
}

void aRTlayer::InsertNewAttrib(SEXP attrValues)
{
	// escolher o nome da TABELA
  	TeAttribute atr;
  	TeDatabase* db = Database;
      
  	SEXP names = getAttrib(attrValues, R_NamesSymbol);
  	string columnName = CHAR(STRING_ELT(names, 0));
  	string errorMessage;
  	atr.rep_.name_= columnName;
  	atr.rep_.type_ = TeREAL;
  	atr.rep_.numChar_ = 6;
  	atr.rep_.decimals_ = 0;
  	//attList.push_back(atr);
  
  	// add a new column
  	if (!db -> addColumn(LayerName, atr.rep_))
	{ // <= ESCOLHER O NOME DA TABELA
    	errorMessage = "Error adding column to table: " + db -> errorMessage();
    	error(errorMessage.c_str());
  	}

  	SEXP vars;
  	char *label = CHAR(STRING_ELT(names, 0));
  	vars = GetListElement(attrValues, label);

  	//add data to new column
  	for (int i = 0; i < length(vars); i++)
	{
    	//string value = CHAR(STRING_ELT(vars, i));
    	double v = REAL(vars)[i];
    	char *value;
    	asprintf(&value, "%f", v);
    	//Rprintf("%f\n", REAL(vars)[0]);
    	char *id;
    	asprintf(&id, "%d", i);

    	string sql = "update " + LayerName + " set " + columnName + " = " + value + " where object_id_ = " + id;
		
    	//string sql = "update pp set newAttr = 0.5";
    	//cout << "sql: " << sql << endl;
    	//Get a portal to the database
    	TeDatabasePortal *portal = db -> getPortal();
    	if (!portal)
      		error("Error trying to get a database portal");

    	db -> execute(sql);
  	}
	return;
}

bool aRTlayer::IsValid()
{
	if(Valid && !Layer())
	{
		Valid = false;
		error("Layer removed from the database!");
	}

	return Valid;
}

